window.originalFetch = window.fetch
window.sessionFetchCache = {}
window.sessionProgressState = {}

function fetchAndReportProgress(url) {
	const outbound = new XMLHttpRequest()

	let defer = {}
	let promise = new Promise((resolve, reject) => {
		defer = { resolve, reject }
	})

	outbound.onprogress = function(event) {
		window.sessionProgressState[url] = event.lengthComputable ? {
			indeterminate: false,
			loaded: event.loaded,
			total: event.total,
			complete: event.loaded === event.total,
			error: null,
		} : {
			indeterminate: true,
			loaded: 0,
			total: 0,
			complete: false,
			error: null
		}
	}
	outbound.onerror = function(event) {
		window.sessionProgressState[url].error = true
		defer.reject(new Error("Failed to get"))
	}
	outbound.onloadstart = function(event) {
		window.sessionProgressState[url] = {
			indeterminate: !event.lengthComputable,
			loaded: event.loaded,
			total: event.total,
			complete: false,
			error: null,
		}
	}
	outbound.onload = function(event) {
		window.sessionProgressState[url] = {
			indeterminate: !event.lengthComputable,
			loaded: event.loaded,
			total: event.total,
			complete: true,
			error: null,
		}
		window.sessionFetchCache[url] = new Uint8Array(outbound.response)
		defer.resolve(window.sessionFetchCache[url])
	}

	outbound.responseType = "arraybuffer"
	outbound.open("GET", url, true)

	outbound.send()

	return promise
}

function checkLoop(url) {
	let defer = {}
	let promise = new Promise((resolve, reject) => {
		defer = { resolve, reject }
	})

	function inner() {
		if (window.sessionFetchCache[url]) {
			defer.resolve(window.sessionFetchCache[url])
		} else if (window.sessionProgressState[url]) {
			const val = window.sessionProgressState[url]
			if (val.error) {
				defer.reject(new Error("Failed to get"))
			} else {
				requestAnimationFrame(inner)
			}
		} else {
			defer.reject(new Error("Missing entry"))
		}

	}

	requestAnimationFrame(inner)

	return promise
}

window.fetch = async function interceptedFetch(...params) {
	if (params.length === 1 && typeof params[0] === "string") {
		const path = params[0]
		if (path.endsWith("wasm")) {
			try {
				const value = await checkLoop(path)
				return new Response(value, {
					headers: {},
					status: 200,
					statusText: 'ok'
				})
			} catch(_) {
				return window.originalFetch(...params)
			}
		}
	}
	return window.originalFetch(...params)
}

window.interceptPreloads = function() {
	let elems = document.querySelectorAll('link[rel="preload"][as="fetch"]')
	const checks = []
	elems.forEach(item => {
		checks.push(item.href)
		fetchAndReportProgress(item.href)
	})

	const start_container = document.getElementById('start_container')
	for (const child of Array.from(start_container.children)) {
		start_container.removeChild(child)
	}

	start_container.innerHTML = `<div style="width: 85%; height: 2.5rem; margin: auto; background-color: lightgray">
	<div id="progressbar" style="width: 0%; height: 2.5rem; background-color: cornflowerblue"></div>
</div>`

	function check_progress() {
		let total_progress = 0
		let total_total = 0

		let is_complete = checks.every(url => window.sessionFetchCache.hasOwnProperty(url))

		if (is_complete) {
			document.getElementById('progressbar').style.width = `100%`
			start_container.innerHTML = document.getElementById('content_loaded').innerHTML
			window.attachStartListener()
		} else {
			for (const url of checks) {
				let value = window.sessionProgressState[url]
				console.log(value)

				if (value) {
					total_progress += value.loaded
					total_total += value.total
				}
			}

			const percent = Math.floor((total_progress / total_total) * 100)
			document.getElementById('progressbar').style.width = `${percent}%`
			requestAnimationFrame(check_progress)
		}
	}

	check_progress()
}
